@@ -1,7 +1,7 @@
 import json
 import os
 import logging
-from service_principal import ServicePrincipal
+from .service_principal import ServicePrincipal
 from azure.identity import DefaultAzureCredential
 from azure.appconfiguration import AzureAppConfigurationClient
 from azure.keyvault.secrets import SecretClient
@@ -9,6 +9,7 @@ from azure.keyvault.secrets import SecretClient
 
 class AppConfiguration:
     """Class for loading Azure App Configurations """
+
     def __init__(self, app_config_name, settings_path=None):
         self.app_config_name = self.get_app_conf_name(app_config_name)
         self.settings_path = self.get_settings_path(settings_path)
@@ -20,8 +21,8 @@ class AppConfiguration:
         settings_path = self.get_settings_path(default_value=settings_path)
         service_principal = ServicePrincipal(settings_path=settings_path)
         service_principal.load_service_principal_into_environment()
-        credential = DefaultAzureCredential()
-
+        credential = DefaultAzureCredential(exclude_shared_token_cache_credential=True,
+                                            exclude_visual_studio_code_credential=False)
         client = AzureAppConfigurationClient(base_url=app_conf_url, credential=credential)
         return client
 
@@ -72,7 +73,7 @@ class AppConfiguration:
 
 # some helper functions for loading settings/secrets
 def load_setting(setting_name, default_value=None, app_config_name=None, settings_path=None,
-                 save_new_config_to_settings_file=False, label=None):
+                 save_new_setting=False, label=None):
     # if default value is provided, just use that
     if default_value is not None:
         return default_value
@@ -89,16 +90,22 @@ def load_setting(setting_name, default_value=None, app_config_name=None, setting
             return setting_value
 
     # if an app config name was provided, try to load from azure
-    app_config = AppConfiguration(app_config_name)
-    app_config_name = app_config.get_app_conf_name(app_config_name)
     if app_config_name is not None:
-        setting_value = app_config.load_app_conf(app_config_name,
-                                                settings_path,
-                                                label=label,
-                                                key=setting_name)
+        app_config = AppConfiguration(app_config_name)
+        app_config_setting = app_config.load_app_conf(app_config_name,
+                                                      settings_path,
+                                                      label=label,
+                                                      key=setting_name)
+        if len(app_config_setting) == 1:
+            setting_value = app_config_setting[0].value
+        else:
+            logging.warning(f'Unable to load setting from app config: {app_config_name}, '
+                            f'key: {setting_name}, label: {label} ')
+            setting_value = None
         if setting_value is not None:
+
             # if we could load the secret from azure check if we want to save it to our settings file
-            if save_new_config_to_settings_file is True and settings_path is not None:
+            if save_new_setting is True and settings_path is not None:
                 new_setting = {setting_name: setting_value}
                 _prepare_settings_file(settings_path, new_setting)
             return setting_value
@@ -147,7 +154,7 @@ def _generate_settings_file(settings_path, settings_template, exit_on_missing_se
         if 'README' in settings_template:
             del settings_template['README']
         f.write(json.dumps(settings_template, indent=2))
-    logging.warning(f'Please open file: {settings_path} and fill out the missing settings')
+    logging.warning(f'Generated settings file {settings_path} with new settings')
     if exit_on_missing_settings:
         exit()
 
@@ -163,3 +170,78 @@ def _load_secret_from_file(secret_name, settings_path):
             logging.debug(f'secret_name: {secret_name} not found in file: {settings_path}')
             return None
         return settings[secret_name]
+
+
+class SettingLoader:
+    """A generic class for loading settings from Env Variables, Local Files, and Azure App Configurations """
+
+    def __init__(self, app_config_name=None, settings_path=None,
+                 save_new_setting=False, label=None):
+
+        self.app_config_name = self.get_app_conf_name(app_config_name)
+        self.settings_path = self.get_settings_path(settings_path)
+        self.save_new_setting = self.get_save_new_setting(save_new_setting)
+        self.label = self.get_label(label)
+
+    def get_app_conf_name(self, default_value):
+        if default_value is not None:
+            return default_value
+        else:
+            try:
+                return self.app_config_name
+            except AttributeError:
+                return None
+
+    def get_settings_path(self, default_value):
+        if default_value is not None:
+            return default_value
+        else:
+            try:
+                return self.settings_path
+            except AttributeError:
+                return None
+
+    def get_save_new_setting(self, default_value):
+        if default_value is not None:
+            return default_value
+        else:
+            try:
+                return self.save_new_setting
+            except AttributeError:
+                return None
+
+    def get_label(self, default_value):
+        if default_value is not None:
+            return default_value
+        else:
+            try:
+                return self.label
+            except AttributeError:
+                return None
+
+    def load_setting(self, setting_name, default_value=None, app_config_name=None, settings_path=None,
+                     save_new_setting=False, label=None):
+
+        app_config_name = self.get_app_conf_name(app_config_name)
+        settings_path = self.get_settings_path(settings_path)
+        save_new_setting = self.get_save_new_setting(save_new_setting)
+        label = self.get_label(label)
+
+        setting_value = load_setting(setting_name,
+                                     default_value=default_value,
+                                     app_config_name=app_config_name,
+                                     settings_path=settings_path,
+                                     save_new_setting=save_new_setting,
+                                     label=label)
+        return setting_value
+
+    def load_setting_into_env(self, setting_name, default_value=None, app_config_name=None, settings_path=None,
+                              save_new_setting=False, label=None):
+        setting_value = self.load_setting(setting_name,
+                                          default_value=default_value,
+                                          app_config_name=app_config_name,
+                                          settings_path=settings_path,
+                                          save_new_setting=save_new_setting,
+                                          label=label)
+        os.environ[setting_name] = setting_value
+        return setting_value
